% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%% This code is used to compare the proposed handover necessity estimation 
%% (HNE) with other handover decision methods, including fixed RSS based 
%% and hysteresis based methods.
clear all

%% Network parameters initialization
R=150;
R_fixed=150;
R_hyst=120;
tau_i=2;
tau_o=2;
P_f=0.02;
P_u=0.04;

%% Velocities of mobile terminals from 1 km/h to 100 km/h are included.
for i=1:100
    v0(i)=i;
    v(i)=v0(i).*5/18;
    
%% Probability of handover failures for the fixed RSS based method.
    if v(i)*tau_i>=2*R_fixed
        P_f_fixed(i)=1;
    else P_f_fixed(i)=2/pi*asin(v(i).*tau_i/2/R_fixed);
    end
    
%% Probability of handover failures for the hysteresis based method.   
    if v(i)*tau_i>=2*R_hyst
        P_f_hyst(i)=1;
    else P_f_hyst(i)=2/pi*asin(v(i).*tau_i/2/R_hyst);
    end 
    
%% Probability of handover failures for HNE.    
    P_f_HNE(i)=P_f;
       
%% Probability of unnecessary handovers for the fixed RSS based method.
    if v(i)*(tau_i+tau_o)>=2*R_fixed
        P_u_fixed(i)=1;
    else P_u_fixed(i)=2/pi*asin(v(i).*(tau_i+tau_o)/2/R_fixed);
    end
    
%% Probability of unnecessary handovers for the hysteresis based method.  
    if v(i)*(tau_i+tau_o)>=2*R_hyst
        P_u_hyst(i)=1;
    else P_u_hyst(i)=2/pi*asin(v(i).*(tau_i+tau_o)/2/R_hyst);
    end 
    
%% Probability of unnecessary handovers for HNE.     
    P_u_HNE(i)=P_u;    
end

%% Plot the figure for probability of handover failures.  
plot(v0,P_f_fixed,'-.b','LineWidth',2);
hold on
plot(v0,P_f_hyst,'--g','LineWidth',2);
hold on
plot(v0,P_f_HNE,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Handover Failure Probability')
%title ('Theoretical Analysis')

%% Plot the figure for probability of unnecessary handovers.  
figure
plot(v0,P_u_fixed,'-.b','LineWidth',2);
hold on
plot(v0,P_u_hyst,'--g','LineWidth',2);
hold on
plot(v0,P_u_HNE,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HNE');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Unnecessary Handover Probability')

%% Calculation of the velocity thresholds.
v_HO1=sin(P_f*pi/2)*2*R_fixed/tau_i*18/5;
v_HO2=sin(P_u*pi/2)*2*R_fixed/(tau_i+tau_o)*18/5;
