% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%% This code is used to compare the proposed handover target selection 
%% (HTS) method with methods which consistently choose one access network, 
%% under different user expected security levels.
clear all

%% User expectations of bandwidth (fixed), monetary cost (fixed) and 
%% security (variable).
exp_bandwidth=500;
exp_money=5;
exp_security=1:10;

%% Network parameters initialization.
bandwidth1=2000;money1=3;security1=1;power1=8;
bandwidth2=1000;money2=2;security2=2;power2=4;
bandwidth3=384;money3=5;security3=7;power3=2;
N_bandwidth = [bandwidth1,bandwidth2,bandwidth3];
N_bandwidth_cost = [1/bandwidth1,1/bandwidth2,1/bandwidth3];
N_money = [money1,money2,money3];
N_security = [security1,security2,security3];
N_security_cost=[1/security1,1/security2,1/security3];
N_power = [power1,power2,power3];

%% Network parameters Normalization.
N_bandwidth_norm = (N_bandwidth_cost)/max(N_bandwidth_cost);
N_money_norm = (N_money)/max(N_money);
N_security_norm = N_security_cost/max(N_security_cost);
N_power_norm = N_power/max(N_power);

%% Weights distribution for HTS.
p_w = 1;
for i=1:length(exp_security)
sum_satisfaction_HTS(i)=0;sum_satisfaction_N1(i)=0;
sum_satisfaction_N2(i)=0;sum_satisfaction_N3(i)=0;
for j=1:1000
i_bandwidth(i,j)=randint(1,1,4);
i_cost(i,j)=randint(1,1,4);
i_security(i,j)=randint(1,1,4);
if i_bandwidth(i,j)~=0 || i_cost(i,j)~=0 || i_security(i,j)~=0
     [ w_bandwidth(i,j),w_cost(i,j),w_security(i,j),w_power(i,j) ] = weights( i_bandwidth(i,j),i_cost(i,j),i_security(i,j),p_w );
else [ w_bandwidth(i,j),w_cost(i,j),w_security(i,j),w_power(i,j) ] = weights( 1,1,1,p_w );
end

%% Handover target selection based on the cost function for HTS.
cost_Net1(i,j)=N_bandwidth_cost(1)*w_bandwidth(i,j)+N_money(1)*w_cost(i,j)+N_security_cost(1)*w_security(i,j)+N_power(1)*w_power(i,j);
cost_Net2(i,j)=N_bandwidth_cost(2)*w_bandwidth(i,j)+N_money(2)*w_cost(i,j)+N_security_cost(2)*w_security(i,j)+N_power(2)*w_power(i,j);
cost_Net3(i,j)=N_bandwidth_cost(3)*w_bandwidth(i,j)+N_money(3)*w_cost(i,j)+N_security_cost(3)*w_security(i,j)+N_power(3)*w_power(i,j);
min_cost(i,j)=min([cost_Net1(i,j),cost_Net2(i,j),cost_Net3(i,j)]);
if min_cost(i,j)==cost_Net1(i,j)
    act_bandwidth(i,j)=N_bandwidth(1);
    act_money(i,j)=N_money(1);
    act_security(i,j)=N_security(1);
elseif min_cost(i,j)==cost_Net2(i,j)
    act_bandwidth(i,j)=N_bandwidth(2);
    act_money(i,j)=N_money(2);
    act_security(i,j)=N_security(2);
elseif min_cost(i,j)==cost_Net3(i,j)
    act_bandwidth(i,j)=N_bandwidth(3);
    act_money(i,j)=N_money(3);
    act_security(i,j)=N_security(3);
end

%% Calculation of user's satisfaction.
satisfaction_HTS(i,j)=(act_bandwidth(i,j)-exp_bandwidth)/act_bandwidth(i,j)*w_bandwidth(i,j) + (exp_money-act_money(i,j))/act_money(i,j)*w_cost(i,j) + (act_security(i,j)-exp_security(i))/act_security(i,j)*w_security(i,j);
satisfaction_N1(i,j)=(N_bandwidth(1)-exp_bandwidth)/N_bandwidth(1)*w_bandwidth(i,j) + (exp_money-N_money(1))/N_money(1)*w_cost(i,j) + (N_security(1)-exp_security(i))/N_security(1)*w_security(i,j);
satisfaction_N2(i,j)=(N_bandwidth(2)-exp_bandwidth)/N_bandwidth(2)*w_bandwidth(i,j) + (exp_money-N_money(2))/N_money(2)*w_cost(i,j) + (N_security(2)-exp_security(i))/N_security(2)*w_security(i,j);
satisfaction_N3(i,j)=(N_bandwidth(3)-exp_bandwidth)/N_bandwidth(3)*w_bandwidth(i,j) + (exp_money-N_money(3))/N_money(3)*w_cost(i,j) + (N_security(3)-exp_security(i))/N_security(3)*w_security(i,j);
sum_satisfaction_HTS(i) = sum_satisfaction_HTS(i)+satisfaction_HTS(i,j);
sum_satisfaction_N1(i) = sum_satisfaction_N1(i)+satisfaction_N1(i,j);
sum_satisfaction_N2(i) = sum_satisfaction_N2(i)+satisfaction_N2(i,j);
sum_satisfaction_N3(i) = sum_satisfaction_N3(i)+satisfaction_N3(i,j);
end
ave_satisfaction_HTS(i) = sum_satisfaction_HTS(i)/1000;
ave_satisfaction_N1(i) = sum_satisfaction_N1(i)/1000;
ave_satisfaction_N2(i) = sum_satisfaction_N2(i)/1000;
ave_satisfaction_N3(i) = sum_satisfaction_N3(i)/1000;
end

%% Plot the figure for user's satisfaction.
figure
plot(exp_security,ave_satisfaction_HTS,'-r','LineWidth',2);
hold on
plot(exp_security,ave_satisfaction_N1,'--g','LineWidth',2);
hold on
plot(exp_security,ave_satisfaction_N2,'-.b','LineWidth',2);
hold on
plot(exp_security,ave_satisfaction_N3,'-y','LineWidth',2);
hold on
legend('HTS','Always choosing network1','Always choosing network2','Always choosing network3');
grid  ('on')
xlabel('Preferred Security Level')
ylabel('User''s Satisfaction')
